searchNodes=[{"ref":"CTE.html","title":"CTE","type":"module","doc":"The Closure Table for Elixir strategy, CTE for short, is a simple and elegant way of storing and working with hierarchies. It involves storing all paths through a tree, not just those with a direct parent-child relationship. You may want to chose this model, over the Nested Sets model, should you need referential integrity and to assign nodes to multiple trees. With CTE you can navigate through hierarchies using a simple API, such as: finding the ascendants and descendants of a node, inserting and deleting nodes, moving entire sub-trees or print them as a digraph (.dot) file. Quick example. For this example we&#39;re using the in-Memory Adapter. This Adapter is useful for prototyping or with data structures that can easily fit in memory; their persistence being taken care of by other components. For more involved use cases, CTE integrates with Ecto using a simple API. When used from a module, the CTE expects the: :otp_app and :adapter attributes, to be defined. The :otp_app should point to an OTP application that might provide additional configuration. Equally so are the: :nodes and the :paths attributes. The :nodes attribute, in the case of the Memory Adapter, is a Map defining your nodes while the: :paths attribute, is a list containing the tree path between the nodes - a list of lists. For example: defmodule CTM do use CTE, otp_app: :ct_empty, adapter: CTE.Adapter.Memory, nodes: %{ 1 =&gt; %{id: 1, author: &quot;Olie&quot;, comment: &quot;Is Closure Table better than the Nested Sets?&quot;}, 2 =&gt; %{id: 2, author: &quot;Rolie&quot;, comment: &quot;It depends. Do you need referential integrity?&quot;}, 3 =&gt; %{id: 3, author: &quot;Polie&quot;, comment: &quot;Yeah.&quot;} }, paths: [[1, 1], [1, 2], [1, 3], [2, 2], [2, 3], [3, 3]] end When using the CTE.Adapter.Ecto, the: :nodes attribute, will be a Schema i.e. Post, TreePath, etc! In our initial implementation, the nodes definitions must have at least the :id, as one of their properties. This caveat will be lifted in a later implementation. Add the CTM module to your main supervision tree: defmodule CTM.Application do @moduledoc false use Application def start(_type, _args) do opts = [strategy: :one_for_one, name: CTM.Supervisor] Supervisor.start_link([CTM], opts) end end Using iex -S mix, for quickly experimenting with the CTE API: find the descendants of comment #1 iex» CTM.descendants(1) {:ok, [3, 2]} find the ancestors iex» CTM.ancestors(2) {:ok, [1]} iex» CTM.ancestors(3) {:ok, [1]} find the ancestors, with information about the node: iex» CTM.ancestors(2, nodes: true) {:ok, [ %{ author: &quot;Olie&quot;, comment: &quot;Is Closure Table better than the Nested Sets?&quot;, id: 1 } ]} move leafs/subtrees around. From being a child of comment #1, to becoming a child of comment #2, in the following example: iex» CTM.move(3, 2, limit: 1) :ok iex» CTM.descendants(2) {:ok, [3]} Please check the docs, the tests, and the examples folder, for more details."},{"ref":"CTE.html#t:config/0","title":"CTE.config/0","type":"type","doc":""},{"ref":"CTE.html#t:nodes/0","title":"CTE.nodes/0","type":"type","doc":""},{"ref":"CTE.html#t:paths/0","title":"CTE.paths/0","type":"type","doc":""},{"ref":"CTE.html#t:repo/0","title":"CTE.repo/0","type":"type","doc":""},{"ref":"CTE.html#t:t/0","title":"CTE.t/0","type":"type","doc":""},{"ref":"CTE.html#t:table/0","title":"CTE.table/0","type":"type","doc":""},{"ref":"CTE.Adapter.html","title":"CTE.Adapter","type":"behaviour","doc":"Specification of the Closure Table implementation. Most of the functions implementing the CTE.Adapter behavior, will accept the following options: :limit, to limit the total number of nodes returned, when finding the ancestors or the descendants for nodes :itself, accepting a boolean value. When true, the node used for finding its neighbors are returned as part of the results. Default: true :nodes, accepting a boolean value. When true, the results are containing additional information about the nodes. Default: false"},{"ref":"CTE.Adapter.html#c:ancestors/3","title":"CTE.Adapter.ancestors/3","type":"callback","doc":"Retrieve the ancestors of a node"},{"ref":"CTE.Adapter.html#c:delete/3","title":"CTE.Adapter.delete/3","type":"callback","doc":"Delete a leaf or a subtree. When limit: 1, the default value, then delete only the leafs, else the entire subtree"},{"ref":"CTE.Adapter.html#c:descendants/3","title":"CTE.Adapter.descendants/3","type":"callback","doc":"Retrieve the descendants of a node"},{"ref":"CTE.Adapter.html#c:insert/4","title":"CTE.Adapter.insert/4","type":"callback","doc":"Insert a node under an existing ancestor"},{"ref":"CTE.Adapter.html#c:move/4","title":"CTE.Adapter.move/4","type":"callback","doc":"Move a subtree from one location to another. First, the subtree and its descendants are disconnected from its ancestors. And second, the subtree is inserted under the new parent (ancestor) and the subtree, including its descendants, is declared as descendants of all the new ancestors."},{"ref":"CTE.Adapter.html#c:tree/3","title":"CTE.Adapter.tree/3","type":"callback","doc":"Calculate and return a &quot;tree&quot; structure containing the paths and the nodes under the given leaf/node"},{"ref":"CTE.Adapter.html#t:options/0","title":"CTE.Adapter.options/0","type":"type","doc":""},{"ref":"CTE.Adapter.html#t:t/0","title":"CTE.Adapter.t/0","type":"type","doc":""},{"ref":"CTE.Adapter.Ecto.html","title":"CTE.Adapter.Ecto","type":"module","doc":"A CTE Adapter implementation using an existing Ecto Repo for persisting the models. The current implementation is depending on Ecto ~&gt; 3.1; using Ecto.SubQuery! For this implementation to work you&#39;ll have to provide two tables, and the name of the Repo used by your application: a table name containing the nodes. Having the id, as a the primary key a table name where the tree paths will be stores. the name of the Ecto.Repo, defined by your app In a future version we will provide you with a convenient migration template to help you starting, but for now you must supply these tables. For example, given you have the following Schemas for comments: defmodule CT.Comment do use Ecto.Schema import Ecto.Changeset @timestamps_opts [type: :utc_datetime] schema &quot;comments&quot; do field :text, :string belongs_to :author, CT.Author timestamps() end end and a table used for storing the parent-child relationships defmodule CT.TreePath do use Ecto.Schema import Ecto.Changeset alias CT.Comment @primary_key false schema &quot;tree_paths&quot; do belongs_to :parent_comment, Comment, foreign_key: :ancestor belongs_to :comment, Comment, foreign_key: :descendant end end we can define the following module: defmodule CT.MyCTE do use CTE, otp_app: :cte, adapter: CTE.Adapter.Ecto, repo: CT.Repo, nodes: CT.Comment, paths: CT.TreePath end We add our CTE Repo to the app&#39;s main supervision tree, like this: defmodule CT.Application do use Application def start(_type, _args) do children = [ CT.Repo, CT.MyCTE ] opts = [strategy: :one_for_one, name: CT.Supervisor] Supervisor.start_link(children, opts) end end restart out app and then using IEx, we can start experimenting. Examples: iex» CT.MyCTE.ancestors(9) {:ok, [1, 4, 6]} iex» CT.MyCTE.tree(6) {:ok, %{ nodes: %{ 6 =&gt; %CT.Comment{ __meta__: #Ecto.Schema.Metadata&lt;:loaded, &quot;comments&quot;&gt;, author: #Ecto.Association.NotLoaded&lt;association :author is not loaded&gt;, author_id: 2, id: 6, inserted_at: ~U[2019-07-21 01:10:35Z], text: &quot;Everything is easier, than with the Nested Sets.&quot;, updated_at: ~U[2019-07-21 01:10:35Z] }, 8 =&gt; %CT.Comment{ __meta__: #Ecto.Schema.Metadata&lt;:loaded, &quot;comments&quot;&gt;, author: #Ecto.Association.NotLoaded&lt;association :author is not loaded&gt;, author_id: 1, id: 8, inserted_at: ~U[2019-07-21 01:10:35Z], text: &quot;I’m sold! And I’ll use its Elixir implementation! &lt;3&quot;, updated_at: ~U[2019-07-21 01:10:35Z] }, 9 =&gt; %CT.Comment{ __meta__: #Ecto.Schema.Metadata&lt;:loaded, &quot;comments&quot;&gt;, author: #Ecto.Association.NotLoaded&lt;association :author is not loaded&gt;, author_id: 3, id: 9, inserted_at: ~U[2019-07-21 01:10:35Z], text: &quot;w⦿‿⦿t!&quot;, updated_at: ~U[2019-07-21 01:10:35Z] } }, paths: [[6, 6], [6, 8], [6, 9], &#39; &#39;, &#39;\b\b&#39;] }} Have fun!"},{"ref":"CTE.Adapter.Ecto.html#child_spec/1","title":"CTE.Adapter.Ecto.child_spec/1","type":"function","doc":"Returns a specification to start this module under a supervisor. See Supervisor."},{"ref":"CTE.Adapter.Ecto.html#init/1","title":"CTE.Adapter.Ecto.init/1","type":"function","doc":"Initializes the adapter supervision tree by returning the children and adapter metadata."},{"ref":"CTE.Adapter.Ecto.html#start_link/1","title":"CTE.Adapter.Ecto.start_link/1","type":"function","doc":"start the Adapter server"},{"ref":"CTE.Adapter.Memory.html","title":"CTE.Adapter.Memory","type":"module","doc":"Basic implementation of the CTE, using the memory for persisting the models. Adapter provided as a convenient way of using CTE in tests or during the development"},{"ref":"CTE.Adapter.Memory.html#child_spec/1","title":"CTE.Adapter.Memory.child_spec/1","type":"function","doc":"Returns a specification to start this module under a supervisor. See Supervisor."},{"ref":"CTE.Adapter.Memory.html#init/1","title":"CTE.Adapter.Memory.init/1","type":"function","doc":"Initializes the adapter supervision tree by returning the children and adapter metadata."},{"ref":"CTE.Adapter.Memory.html#start_link/1","title":"CTE.Adapter.Memory.start_link/1","type":"function","doc":"start the Adapter server"},{"ref":"CTE.Utils.html","title":"CTE.Utils","type":"module","doc":"Basic utilities for helping developing functionality about the CTE data structures"},{"ref":"CTE.Utils.html#print_dot/2","title":"CTE.Utils.print_dot/2","type":"function","doc":"render a path hierarchy as a .dot string that you could use for drawing your results, using graphviz. w⦿‿⦿t! Upon receiving something like this: %{ nodes: %{ 6 =&gt; %{ author: &quot;Rolie&quot;, comment: &quot;Everything is easier, than with the Nested Sets.&quot;, id: 6 }, 8 =&gt; %{ author: &quot;Olie&quot;, comment: &quot;I’m sold! And I’ll use its Elixir implementation! &lt;3&quot;, id: 8 }, 9 =&gt; %{author: &quot;Polie&quot;, comment: &quot;w⦿‿⦿t!&quot;, id: 9} }, paths: [[6, 6], [6, 8], [6, 9], &#39; &#39;, &#39;\b\b&#39;] } this will output a .dot formatted string that you could use later for generating an image: dot -Tpng &lt;filename&gt;.dot -o &lt;filename&gt;.png"},{"ref":"readme.html","title":"Closure Table","type":"extras","doc":"Closure Table while an early preview, this library provides two adapters: an in-memory one, and one for using the closure-table solution with Ecto; for your testing and development convenience. The Closure Table solution is a simple and elegant way of storing hierarchies. It involves storing all paths through a tree, not just those with a direct parent-child relationship. You may want to chose this model, over the Nested Sets model, should you need referential integrity and to assign nodes to multiple trees. Throughout the various examples and tests, we will refer to the hierarchies depicted below, where we&#39;re modeling a hypothetical forum-like discussion between Rolie, Olie and Polie, and their debate around the usefulness of this implementation :)"},{"ref":"readme.html#quick-start","title":"Closure Table - Quick start","type":"extras","doc":"To start, you can simply use one Adapter from the ones provided, same way you&#39;d use the Ecto&#39;s own Repo: defmodule CTM do use CTE, otp_app: :ct_empty, adapter: CTE.Adapter.Memory, nodes: %{ 1 =&gt; %{id: 1, author: &quot;Olie&quot;, comment: &quot;Is Closure Table better than the Nested Sets?&quot;}, 2 =&gt; %{id: 2, author: &quot;Rolie&quot;, comment: &quot;It depends. Do you need referential integrity?&quot;}, 3 =&gt; %{id: 3, author: &quot;Polie&quot;, comment: &quot;Yeah.&quot;} }, paths: [[1, 1], [1, 2], [1, 3], [2, 2], [2, 3], [3, 3]] end With the configuration above, the :nodes attribute is a map containing the comments our interlocutors made; these are &quot;nodes&quot;, in CTE&#39;s parlance. When using the CTE.Adapter.Ecto implementation, the :nodes attribute will be a Schema (or a table name! In our initial implementation, the nodes definitions must have at least the :id, as one of their properties. This caveat will be lifted in a later implementation. The :paths attribute represents the parent-child relationship between the comments. Add the CTM module to your main supervision tree: defmodule CTM.Application do @moduledoc false use Application def start(_type, _args) do opts = [strategy: :one_for_one, name: CTM.Supervisor] Supervisor.start_link([CTM], opts) end end And then using iex -S mix, for quickly experimenting with the CTE API, let&#39;s find the descendants of comment #1: iex» CTM.descendants(1) {:ok, [3, 2]} Please check the docs for more details and return from more updates! Oh and there is a simple utility for helping you drawing your paths, using graphviz! From Rolie&#39;s comments, excerpt: Maybe useful?! If yes, then we&#39;ll let you find this function by yourself ;) hint: check the tests &lt;3"},{"ref":"readme.html#installation","title":"Closure Table - Installation","type":"extras","doc":"If available in Hex, the package can be installed by adding closure_table to your list of dependencies in mix.exs: def deps do [ {:closure_table, &quot;~&gt; 0.1&quot;} ] end"},{"ref":"readme.html#contributing","title":"Closure Table - Contributing","type":"extras","doc":"Fork this project Create your feature branch (git checkout -b my-new-feature) Test (mix test) Commit your changes (git commit -am &#39;Add some feature&#39;) Push to the branch (git push origin my-new-feature) Create new Pull Request"},{"ref":"readme.html#license","title":"Closure Table - License","type":"extras","doc":"Copyright 2019 Florin T.PATRASCU Licensed under the Apache License, Version 2.0 (the &quot;License&quot;); you may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License."},{"ref":"changelog.html","title":"CHANGELOG","type":"extras","doc":"CHANGELOG"},{"ref":"changelog.html#0-1-0","title":"CHANGELOG - 0.1.0","type":"extras","doc":"initial version"},{"ref":"changelog.html#0-1-1","title":"CHANGELOG - 0.1.1","type":"extras","doc":"minor text adjustments removing redundant content"}]